package javax.swing;

import java.awt.Paint;
import javax.swing.border.StrokeBorder;
import java.awt.BasicStroke;
import javax.swing.border.MatteBorder;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import java.awt.Font;
import javax.swing.border.TitledBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.SoftBevelBorder;
import javax.swing.border.BevelBorder;
import javax.swing.border.LineBorder;
import java.awt.Color;
import javax.swing.border.Border;

public class BorderFactory
{
    static final Border sharedRaisedBevel;
    static final Border sharedLoweredBevel;
    private static Border sharedSoftRaisedBevel;
    private static Border sharedSoftLoweredBevel;
    static final Border sharedEtchedBorder;
    private static Border sharedRaisedEtchedBorder;
    static final Border emptyBorder;
    private static Border sharedDashedBorder;
    
    private BorderFactory() {
    }
    
    public static Border createLineBorder(Color color) {
        return new LineBorder(color, 1);
    }
    
    public static Border createLineBorder(Color color, int thickness) {
        return new LineBorder(color, thickness);
    }
    
    public static Border createLineBorder(Color color, int thickness, boolean roundedCorners) {
        return new LineBorder(color, thickness, roundedCorners);
    }
    
    public static Border createRaisedBevelBorder() {
        return createSharedBevel(0);
    }
    
    public static Border createLoweredBevelBorder() {
        return createSharedBevel(1);
    }
    
    public static Border createBevelBorder(int n) {
        return createSharedBevel(n);
    }
    
    public static Border createBevelBorder(int bevelType, Color highlight, Color shadow) {
        return new BevelBorder(bevelType, highlight, shadow);
    }
    
    public static Border createBevelBorder(int bevelType, Color highlightOuterColor, Color highlightInnerColor, Color shadowOuterColor, Color shadowInnerColor) {
        return new BevelBorder(bevelType, highlightOuterColor, highlightInnerColor, shadowOuterColor, shadowInnerColor);
    }
    
    static Border createSharedBevel(int n) {
        if (n == 0) {
            return BorderFactory.sharedRaisedBevel;
        }
        if (n == 1) {
            return BorderFactory.sharedLoweredBevel;
        }
        return null;
    }
    
    public static Border createRaisedSoftBevelBorder() {
        if (BorderFactory.sharedSoftRaisedBevel == null) {
            BorderFactory.sharedSoftRaisedBevel = new SoftBevelBorder(0);
        }
        return BorderFactory.sharedSoftRaisedBevel;
    }
    
    public static Border createLoweredSoftBevelBorder() {
        if (BorderFactory.sharedSoftLoweredBevel == null) {
            BorderFactory.sharedSoftLoweredBevel = new SoftBevelBorder(1);
        }
        return BorderFactory.sharedSoftLoweredBevel;
    }
    
    public static Border createSoftBevelBorder(int n) {
        if (n == 0) {
            return createRaisedSoftBevelBorder();
        }
        if (n == 1) {
            return createLoweredSoftBevelBorder();
        }
        return null;
    }
    
    public static Border createSoftBevelBorder(int bevelType, Color highlight, Color shadow) {
        return new SoftBevelBorder(bevelType, highlight, shadow);
    }
    
    public static Border createSoftBevelBorder(int bevelType, Color highlightOuterColor, Color highlightInnerColor, Color shadowOuterColor, Color shadowInnerColor) {
        return new SoftBevelBorder(bevelType, highlightOuterColor, highlightInnerColor, shadowOuterColor, shadowInnerColor);
    }
    
    public static Border createEtchedBorder() {
        return BorderFactory.sharedEtchedBorder;
    }
    
    public static Border createEtchedBorder(Color highlight, Color shadow) {
        return new EtchedBorder(highlight, shadow);
    }
    
    public static Border createEtchedBorder(int n) {
        switch (n) {
            case 0: {
                if (BorderFactory.sharedRaisedEtchedBorder == null) {
                    BorderFactory.sharedRaisedEtchedBorder = new EtchedBorder(0);
                }
                return BorderFactory.sharedRaisedEtchedBorder;
            }
            case 1: {
                return BorderFactory.sharedEtchedBorder;
            }
            default: {
                throw new IllegalArgumentException("type must be one of EtchedBorder.RAISED or EtchedBorder.LOWERED");
            }
        }
    }
    
    public static Border createEtchedBorder(int etchType, Color highlight, Color shadow) {
        return new EtchedBorder(etchType, highlight, shadow);
    }
    
    public static TitledBorder createTitledBorder(String title) {
        return new TitledBorder(title);
    }
    
    public static TitledBorder createTitledBorder(Border border) {
        return new TitledBorder(border);
    }
    
    public static TitledBorder createTitledBorder(Border border, String title) {
        return new TitledBorder(border, title);
    }
    
    public static TitledBorder createTitledBorder(Border border, String title, int titleJustification, int titlePosition) {
        return new TitledBorder(border, title, titleJustification, titlePosition);
    }
    
    public static TitledBorder createTitledBorder(Border border, String title, int titleJustification, int titlePosition, Font titleFont) {
        return new TitledBorder(border, title, titleJustification, titlePosition, titleFont);
    }
    
    public static TitledBorder createTitledBorder(Border border, String title, int titleJustification, int titlePosition, Font titleFont, Color titleColor) {
        return new TitledBorder(border, title, titleJustification, titlePosition, titleFont, titleColor);
    }
    
    public static Border createEmptyBorder() {
        return BorderFactory.emptyBorder;
    }
    
    public static Border createEmptyBorder(int top, int left, int bottom, int right) {
        return new EmptyBorder(top, left, bottom, right);
    }
    
    public static CompoundBorder createCompoundBorder() {
        return new CompoundBorder();
    }
    
    public static CompoundBorder createCompoundBorder(Border outsideBorder, Border insideBorder) {
        return new CompoundBorder(outsideBorder, insideBorder);
    }
    
    public static MatteBorder createMatteBorder(int top, int left, int bottom, int right, Color matteColor) {
        return new MatteBorder(top, left, bottom, right, matteColor);
    }
    
    public static MatteBorder createMatteBorder(int top, int left, int bottom, int right, Icon tileIcon) {
        return new MatteBorder(top, left, bottom, right, tileIcon);
    }
    
    public static Border createStrokeBorder(BasicStroke stroke) {
        return new StrokeBorder(stroke);
    }
    
    public static Border createStrokeBorder(BasicStroke stroke, Paint paint) {
        return new StrokeBorder(stroke, paint);
    }
    
    public static Border createDashedBorder(Paint paint) {
        return createDashedBorder(paint, 1.0f, 1.0f, 1.0f, false);
    }
    
    public static Border createDashedBorder(Paint paint, float n, float n2) {
        return createDashedBorder(paint, 1.0f, n, n2, false);
    }
    
    public static Border createDashedBorder(Paint paint, float width, float n, float n2, boolean join) {
        int n3 = !join && paint == null && width == 1.0f && n == 1.0f && n2 == 1.0f;
        if (n3 && BorderFactory.sharedDashedBorder != null) {
            return BorderFactory.sharedDashedBorder;
        }
        if (width < 1.0f) {
            throw new IllegalArgumentException("thickness is less than 1");
        }
        if (n < 1.0f) {
            throw new IllegalArgumentException("length is less than 1");
        }
        if (n2 < 0.0f) {
            throw new IllegalArgumentException("spacing is less than 0");
        }
        Border strokeBorder = createStrokeBorder(new BasicStroke(width, join ? 1 : 2, join?1:0, width * 2.0f, new float[] { width * (n - 1.0f), width * (n2 + 1.0f) }, 0.0f), paint);
        if (!(n3 == 0)) {
            BorderFactory.sharedDashedBorder = strokeBorder;
            return strokeBorder;
        }
        return strokeBorder;
    }
    
    static {
        sharedRaisedBevel = new BevelBorder(0);
        sharedLoweredBevel = new BevelBorder(1);
        sharedEtchedBorder = new EtchedBorder();
        emptyBorder = new EmptyBorder(0, 0, 0, 0);
    }
}
